﻿using CrmField.Win8App.Common;
using CrmField.Win8App.Data;
using CRMField.Win8App.Common;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Popups;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;

// The Grouped Items Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234231

namespace CrmField.Win8App
{
    /// <summary>
    /// A page that displays a grouped collection of items.
    /// </summary>
    public sealed partial class GroupedItemsPage : CrmField.Win8App.Common.LayoutAwarePage
    {
        public GroupedItemsPage()
        {
            this.InitializeComponent();
        }

        /// <summary>
        /// Populates the page with content passed during navigation.  Any saved state is also
        /// provided when recreating a page from a prior session.
        /// </summary>
        /// <param name="navigationParameter">The parameter value passed to
        /// <see cref="Frame.Navigate(Type, Object)"/> when this page was initially requested.
        /// </param>
        /// <param name="pageState">A dictionary of state preserved by this page during an earlier
        /// session.  This will be null the first time a page is visited.</param>
        protected override async void LoadState(Object navigationParameter, Dictionary<String, Object> pageState)
        {
            this.DefaultViewModel["CrmContext"] = CrmContextManager.Context; //Added by MA

            //Check the CRM Connection
            if (!CrmContextManager.IsValid())
            {
                CrmSettingsDialog.IsOpen = true;
            }
            else
            {              

                if (ActivityDataSource.CurrentDataSource == null)
                {
                    try
                    {
                        await ActivityDataSource.LoadFromFileOrService(App.DataSourceName);
                    }
                    catch (Exception ex)
                    {
                        var messageDialog = new MessageDialog("Could not Load Data. \n" + ex.Message);
                        messageDialog.ShowAsync();
                    }
                }

                if (WnsChannelManager.Channel == null)
                {
                    try
                    {
                        await WnsChannelManager.SetupChannel();
                    }
                    catch (Exception ex)
                    {
                        var messageDialog = new MessageDialog("Error creating WNS Channel. \n" + ex.Message);
                        messageDialog.ShowAsync();
                    }
                }

                this.DefaultViewModel["DataSource"] = ActivityDataSource.CurrentDataSource;                
            }

            ProgressRingMain.IsActive = false;
        }

        private void Button_Click_Connect(object sender, RoutedEventArgs e)
        {
            CrmSettingsDialog.IsOpen = false;
            this.LoadState("AllGroups", null);
            ProgressRingMain.IsActive = true;
        }

        /// <summary>
        /// Invoked when an item within a group is clicked.
        /// </summary>
        /// <param name="sender">The GridView (or ListView when the application is snapped)
        /// displaying the item clicked.</param>
        /// <param name="e">Event data that describes the item clicked.</param>
        void ItemView_ItemClick(object sender, ItemClickEventArgs e)
        {
            // Navigate to the appropriate destination page, configuring the new page
            // by passing required information as a navigation parameter
            var itemId = ((ServiceAppointment)e.ClickedItem).ActivityId;
            this.Frame.Navigate(typeof(ItemDetailPage), itemId);
        }

        private async void Button_Click_Refresh(object sender, RoutedEventArgs e)
        {
            ProgressRingMain.IsActive = true;
            this.DefaultViewModel["DataSource"] = null;
            await ActivityDataSource.CurrentDataSource.SyncWithService();
            this.DefaultViewModel["DataSource"] = ActivityDataSource.CurrentDataSource;
            ProgressRingMain.IsActive = false;
        }
    }
}
