// =====================================================================
//
//  This file is part of the Microsoft Dynamics CRM SDK code samples.
//
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  This source code is intended only as a supplement to Microsoft
//  Development Tools and/or on-line documentation.  See these other
//  materials for detailed information regarding Microsoft code samples.
//
//  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
// =====================================================================
using System;
using System.Diagnostics.CodeAnalysis;
using System.Runtime.Serialization;
using Microsoft.Xrm.Sdk.Temporary;

namespace Microsoft.Crm.Sdk.Messages
{
	/// <summary>
	/// Defines a collection of appointments to be ignored for a specified resource
	/// during searching for availability in Services layer.
	/// </summary>
	[DataContract(Namespace = "http://schemas.microsoft.com/crm/2011/Contracts")]
	public sealed class AppointmentsToIgnore : IExtensibleDataObject
	{
		/// <summary>
		/// Even though this constructor does not do anything, it is still needed because 
		/// XmlSerializer requires public default constructor on the class to be serialized.
		/// </summary>
		public AppointmentsToIgnore()
		{
		}

		/// <summary>
		/// Initializes an AppointmentsToIgnore object.
		/// </summary>
		/// <param name="appointments">Array of Ids of appointments to be ignored 
		/// for a specified resource.</param>
		/// <param name="resourceId">Id of a resource.</param>
		public AppointmentsToIgnore(Guid[] appointments, Guid resourceId)
		{
			this.Appointments = appointments;
			this.ResourceId = resourceId;
		}

		/// <summary>
		/// Array of Ids of appointments to be ignored for a specified resource.
		/// </summary>
		[DataMember]
		[SuppressMessage("Microsoft.Performance", "CA1819:PropertiesShouldNotReturnArrays", Justification = "Needed to ensure backward compatibility")]
		public Guid[] Appointments { get; set; }

		/// <summary>
		/// Id of a resource.
		/// </summary>
		[DataMember]
		public Guid ResourceId { get; set; }


		#region IExtensibleDataObject Implementation
		private ExtensionDataObject _extensionDataObject;
		public ExtensionDataObject ExtensionData
		{
			get
			{
				return _extensionDataObject;
			}
			set
			{
				_extensionDataObject = value;
			}
		}
		#endregion IExtensibleDataObject Implementation
	}
}
