// =====================================================================
//
//  This file is part of the Microsoft Dynamics CRM SDK code samples.
//
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  This source code is intended only as a supplement to Microsoft
//  Development Tools and/or on-line documentation.  See these other
//  materials for detailed information regarding Microsoft code samples.
//
//  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
// =====================================================================
using System.Runtime.Serialization;

using Microsoft.Xrm.Sdk;
using Microsoft.Xrm.Sdk.Temporary;

namespace Microsoft.Crm.Sdk.Messages
{
	/// <summary>
	/// Collection of objects of AuditPartitionDetails class. 
	/// Used in returning the change data partition details of a given audit record
	/// </summary>
	[CollectionDataContract(Namespace = "http://schemas.microsoft.com/crm/2011/Contracts")]
	public sealed class AuditPartitionDetailCollection : DataCollection<AuditPartitionDetail>, IExtensibleDataObject
	{
		/// <summary>
		/// Indicates if the partition list is logical. The list is logical when the SQL Server
		/// edition doesn't support partitions. Currently partitioning feature is available only
		/// on enterprise edition of SQL Server.
		/// </summary>
		[DataMember]
		public bool IsLogicalCollection { get; set; }

		#region IExtensibleDataObject Implementation
		private ExtensionDataObject _extensionDataObject;
		public ExtensionDataObject ExtensionData
		{
			get
			{
				return _extensionDataObject;
			}
			set
			{
				_extensionDataObject = value;
			}
		}
		#endregion IExtensibleDataObject Implementation
	}
}

