// =====================================================================
//
//  This file is part of the Microsoft Dynamics CRM SDK code samples.
//
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  This source code is intended only as a supplement to Microsoft
//  Development Tools and/or on-line documentation.  See these other
//  materials for detailed information regarding Microsoft code samples.
//
//  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
// =====================================================================
using System;
using System.Runtime.Serialization;
using Microsoft.Xrm.Sdk.Temporary;

namespace Microsoft.Crm.Sdk.Messages
{
	/// <summary>
	/// Defines a constraint and its association to an engine item.
	/// </summary>
	[DataContract(Namespace = "http://schemas.microsoft.com/crm/2011/Contracts")]
	public sealed class ConstraintRelation : IExtensibleDataObject
	{
		/// <summary>
		/// Even though this constructor does not do anything, it is still needed because 
		/// XmlSerializer requires public default constructor on the class to be serialized.
		/// </summary>
		public ConstraintRelation()
		{
		}

		/// <summary>
		/// Initializes a ConstraintRelation object.
		/// </summary>
		/// <param name="objectId">Id of the object this constraint operates on.</param>
		/// <param name="constraintType">Type of constraint (ex. resource selection).</param>
		/// <param name="constraints">Array of constraints.</param>
		public ConstraintRelation(Guid objectId, string constraintType, string constraints)
		{
			this.ObjectId = objectId;
			this.ConstraintType = constraintType;
			this.Constraints = constraints;
		}

		/// <summary>
		/// Id of the object the constraint operates on.
		/// </summary>
		[DataMember]
		public Guid ObjectId { get; set; }

		/// <summary>
		/// Type of constraint, such as preceeds, follows, or resource specification.
		/// </summary>
		[DataMember]
		public string ConstraintType { get; set; }

		/// <summary>
		/// Expression describing the constraint that operates on a set of parameters 
		/// consistent with the constraint type.
		/// </summary>
		[DataMember]
		public string Constraints { get; set; }

		#region IExtensibleDataObject Implementation
		private ExtensionDataObject _extensionDataObject;
		public ExtensionDataObject ExtensionData
		{
			get
			{
				return _extensionDataObject;
			}
			set
			{
				_extensionDataObject = value;
			}
		}
		#endregion IExtensibleDataObject Implementation
	}
}
