// =====================================================================
//
//  This file is part of the Microsoft Dynamics CRM SDK code samples.
//
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  This source code is intended only as a supplement to Microsoft
//  Development Tools and/or on-line documentation.  See these other
//  materials for detailed information regarding Microsoft code samples.
//
//  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
// =====================================================================
using System.Diagnostics.CodeAnalysis;
using System.Runtime.Serialization;
using Microsoft.Xrm.Sdk.Temporary;

namespace Microsoft.Crm.Sdk.Messages
{
	/// <summary>
	/// Error information for each error code.
	/// </summary>
	[DataContract(Namespace = "http://schemas.microsoft.com/crm/2011/Contracts")]
	public sealed class ErrorInfo : IExtensibleDataObject
	{
		/// <summary>
		/// Default constructor.
		/// </summary>
		public ErrorInfo()
		{
			this.ResourceList = new ResourceInfo[0];
			this.ErrorCode = "";
		}

		/// <summary>
		/// Constructor.
		/// </summary>
		/// <param name="errorCode">Error code.</param>
		/// <param name="resourceList">Resource list associated with the error code.</param>
		public ErrorInfo(string errorCode, ResourceInfo[] resourceList)
		{
			this.ResourceList = resourceList;
			this.ErrorCode = errorCode;
		}

		/// <summary>
		/// Get resource list for this error code.
		/// </summary>
		[DataMember]
		[SuppressMessage("Microsoft.Performance", "CA1819:PropertiesShouldNotReturnArrays", Justification = "Needed to ensure backward compatibility")]
		public ResourceInfo[] ResourceList { get; set; }

		/// <summary>
		/// Get error code.
		/// </summary>
		[DataMember]
		public string ErrorCode { get; set; }

		#region IExtensibleDataObject Implementation
		private ExtensionDataObject _extensionDataObject;
		public ExtensionDataObject ExtensionData
		{
			get
			{
				return _extensionDataObject;
			}
			set
			{
				_extensionDataObject = value;
			}
		}
		#endregion IExtensibleDataObject Implementation
	}
}
