// =====================================================================
//
//  This file is part of the Microsoft Dynamics CRM SDK code samples.
//
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  This source code is intended only as a supplement to Microsoft
//  Development Tools and/or on-line documentation.  See these other
//  materials for detailed information regarding Microsoft code samples.
//
//  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
// =====================================================================
using System;
using System.Runtime.Serialization;
using Microsoft.Xrm.Sdk.Temporary;

namespace Microsoft.Crm.Sdk.Messages
{
	/// <summary>
	/// Defines an objective, and its relationship to a resource specificaiton.
	/// </summary>
	[DataContract(Namespace = "http://schemas.microsoft.com/crm/2011/Contracts")]
	public sealed class ObjectiveRelation : IExtensibleDataObject
	{
		/// <summary>
		/// Even though this constructor does not do anything, it is still needed because 
		/// XmlSerializer requires public default constructor on the class to be serialized.
		/// </summary>
		public ObjectiveRelation()
		{
		}

		/// <summary>
		/// Initializes an ObjectiveRelation object.
		/// </summary>
		/// <param name="resourceSpecId">Id of the resource specificaiton 
		/// that this objective applies to.</param>
		/// <param name="objectiveExpression">Binding objective expression.</param>
		public ObjectiveRelation(Guid resourceSpecId, string objectiveExpression)
		{
			this.ResourceSpecId = resourceSpecId;
			this.ObjectiveExpression = objectiveExpression;
		}

		/// <summary>
		/// Id of the resource specificiation this objective applies to.
		/// </summary>
		[DataMember]
		public Guid ResourceSpecId { get; set; }

		/// <summary>
		/// Expression describing the objective. This expression must take
		/// resource as a parameter and return a double.
		/// </summary>
		[DataMember]
		public string ObjectiveExpression { get; set; }

		#region IExtensibleDataObject Implementation
		private ExtensionDataObject _extensionDataObject;
		public ExtensionDataObject ExtensionData
		{
			get
			{
				return _extensionDataObject;
			}
			set
			{
				_extensionDataObject = value;
			}
		}
		#endregion IExtensibleDataObject Implementation
	}
}
