// =====================================================================
//
//  This file is part of the Microsoft Dynamics CRM SDK code samples.
//
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  This source code is intended only as a supplement to Microsoft
//  Development Tools and/or on-line documentation.  See these other
//  materials for detailed information regarding Microsoft code samples.
//
//  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
// =====================================================================
using System;
using System.Runtime.Serialization;
using Microsoft.Xrm.Sdk.Temporary;

namespace Microsoft.Crm.Sdk.Messages
{
	/// <summary>
	/// Defines a resource required in AppointmentRequest.
	/// </summary>
	[DataContract(Namespace = "http://schemas.microsoft.com/crm/2011/Contracts")]
	public sealed class RequiredResource : IExtensibleDataObject
	{
		/// <summary>
		/// Even though this constructor does not do anything, it is still needed because 
		/// XmlSerializer requires public default constructor on the class to be serialized.
		/// </summary>
		public RequiredResource()
		{
		}

		/// <summary>
		/// Initializes a RequiredResource object. 
		/// </summary>
		/// <param name="resourceId">ResourceId of this required resource.</param>
		/// <param name="resourceSpecId">ResourceSpecificationId of the parent ResourceSpecification 
		/// of this required resource.</param>
		public RequiredResource(Guid resourceId, Guid resourceSpecId)
		{
			this.ResourceId = resourceId;
			this.ResourceSpecId = resourceSpecId;
		}

		/// <summary>
		/// ResourceId of this required resource.
		/// </summary>
		[DataMember]
		public Guid ResourceId { get; set; }

		/// <summary>
		/// ResourceSpecificationId of the parent ResourceSpecification of this required resource.
		/// </summary>
		[DataMember]
		public Guid ResourceSpecId { get; set; }

		#region IExtensibleDataObject Implementation
		private ExtensionDataObject _extensionDataObject;
		public ExtensionDataObject ExtensionData
		{
			get
			{
				return _extensionDataObject;
			}
			set
			{
				_extensionDataObject = value;
			}
		}
		#endregion IExtensibleDataObject Implementation
	}
}
