// =====================================================================
//
//  This file is part of the Microsoft Dynamics CRM SDK code samples.
//
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  This source code is intended only as a supplement to Microsoft
//  Development Tools and/or on-line documentation.  See these other
//  materials for detailed information regarding Microsoft code samples.
//
//  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
// =====================================================================
using System;
using System.Runtime.Serialization;
using Microsoft.Xrm.Sdk.Temporary;

namespace Microsoft.Crm.Sdk.Messages
{

	/// <summary>
	/// Resource information
	/// </summary>
	[DataContract(Namespace = "http://schemas.microsoft.com/crm/2011/Contracts")]
	public sealed class ResourceInfo : IExtensibleDataObject
	{
		/// <summary>
		/// Default Constructor for serialization
		/// </summary>
		public ResourceInfo()
		{
			this.Id = Guid.Empty;
			this.DisplayName = null;
		}

		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="id">Resource Id.</param>
		/// <param name="objectTypeCode">Type code of resource.</param>
		/// <param name="name">Resource Name.</param>
		public ResourceInfo(Guid id, string entityName, string name)
		{
			this.Id = id;
			this.EntityName = entityName;
			this.DisplayName = name;
		}

		/// <summary>
		/// Id of resource
		/// </summary>
		[DataMember]
		public Guid Id { get; set; }

		/// <summary>
		/// Name of resource
		/// </summary>
		[DataMember]
		public string DisplayName { get; set; }

		/// <summary>
		/// Name of entity type.
		/// </summary>
		[DataMember]
		public string EntityName { get; set; }


		#region IExtensibleDataObject Implementation
		private ExtensionDataObject _extensionDataObject;
		public ExtensionDataObject ExtensionData
		{
			get
			{
				return _extensionDataObject;
			}
			set
			{
				_extensionDataObject = value;
			}
		}
		#endregion IExtensibleDataObject Implementation
	}
}
