// =====================================================================
//
//  This file is part of the Microsoft Dynamics CRM SDK code samples.
//
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  This source code is intended only as a supplement to Microsoft
//  Development Tools and/or on-line documentation.  See these other
//  materials for detailed information regarding Microsoft code samples.
//
//  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
// =====================================================================
using System;
using System.Runtime.Serialization;
using Microsoft.Xrm.Sdk.Temporary;

namespace Microsoft.Crm.Sdk.Messages
{
	[DataContract(Namespace = "http://schemas.microsoft.com/crm/2011/Contracts")]
	public enum PrivilegeDepth
	{
		[EnumMember]
		Basic = 0,

		[EnumMember]
		Local = 1,

		[EnumMember]
		Deep = 2,

		[EnumMember]
		Global = 3,
	}

	[DataContract(Namespace = "http://schemas.microsoft.com/crm/2011/Contracts")]
	public sealed class RolePrivilege : IExtensibleDataObject
	{
		public RolePrivilege()
		{
		}

		public RolePrivilege(int depth, Guid privilegeId)
		{
			Depth = (PrivilegeDepth)depth;
			PrivilegeId = privilegeId;
			BusinessUnitId = Guid.Empty;
		}

		public RolePrivilege(int depth, Guid privilegeId, Guid businessId)
			: this((PrivilegeDepth)depth, privilegeId, businessId)
		{
		}

		public RolePrivilege(PrivilegeDepth depth, Guid privilegeId, Guid businessId)
		{
			Depth = depth;
			PrivilegeId = privilegeId;
			BusinessUnitId = businessId;
		}

		[DataMember]
		public PrivilegeDepth Depth { get; set; }

		[DataMember]
		public Guid PrivilegeId { get; set; }

		[DataMember]
		public Guid BusinessUnitId { get; set; }

		#region IExtensibleDataObject Implementation
		private ExtensionDataObject _extensionDataObject;
		public ExtensionDataObject ExtensionData
		{
			get
			{
				return _extensionDataObject;
			}
			set
			{
				_extensionDataObject = value;
			}
		}
		#endregion IExtensibleDataObject Implementation
	}
}
