// =====================================================================
//
//  This file is part of the Microsoft Dynamics CRM SDK code samples.
//
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  This source code is intended only as a supplement to Microsoft
//  Development Tools and/or on-line documentation.  See these other
//  materials for detailed information regarding Microsoft code samples.
//
//  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
// =====================================================================
using System.Diagnostics.CodeAnalysis;
using System.Runtime.Serialization;
using Microsoft.Xrm.Sdk.Temporary;

namespace Microsoft.Crm.Sdk.Messages
{
	/// <summary>
	///  Search results with possible trace information
	/// </summary>
	[DataContract(Namespace = "http://schemas.microsoft.com/crm/2011/Contracts")]
	public sealed class SearchResults : IExtensibleDataObject
	{
		/// <summary>
		/// Default constructor.
		/// </summary>
		public SearchResults()
		{
			this.Proposals = new AppointmentProposal[0];
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="proposals">Appointment proposals.</param>
		/// <param name="traceInfo">Trace information.</param>
		public SearchResults(AppointmentProposal[] proposals, TraceInfo traceInfo)
		{
			this.Proposals = proposals;
			this.TraceInfo = traceInfo;
		}

		/// <summary>
		/// Proposals accessor.
		/// </summary>
		[DataMember]
		[SuppressMessage("Microsoft.Performance", "CA1819:PropertiesShouldNotReturnArrays", Justification = "Needed to ensure backward compatibility")]
		public AppointmentProposal[] Proposals { get; set; }

		/// <summary>
		/// Trace information
		/// </summary>
		[DataMember]
		public TraceInfo TraceInfo { get; set; }

		#region IExtensibleDataObject Implementation
		private ExtensionDataObject _extensionDataObject;
		public ExtensionDataObject ExtensionData
		{
			get
			{
				return _extensionDataObject;
			}
			set
			{
				_extensionDataObject = value;
			}
		}
		#endregion IExtensibleDataObject Implementation
	}
}
