// =====================================================================
//
//  This file is part of the Microsoft Dynamics CRM SDK code samples.
//
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  This source code is intended only as a supplement to Microsoft
//  Development Tools and/or on-line documentation.  See these other
//  materials for detailed information regarding Microsoft code samples.
//
//  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
// =====================================================================
using System;
using System.Runtime.Serialization;
using Microsoft.Xrm.Sdk.Temporary;

namespace Microsoft.Crm.Sdk.Messages
{
	/// <summary>
	/// Book result with trace information.
	/// </summary>
	[DataContract(Namespace = "http://schemas.microsoft.com/crm/2011/Contracts")]
	public sealed class ValidationResult : IExtensibleDataObject
	{
		/// <summary>
		/// Default constructor for searialization.
		/// </summary>
		public ValidationResult()
		{
			this.ValidationSuccess = false;
			this.TraceInfo = null;
		}

		/// <summary>
		/// Initiaializes an instance of Book/Validate/ReSchedule result.
		/// </summary>
		/// <param name="success">Success or not</param>
		/// <param name="traceInfo">Trace information.</param>
		/// <param name="activityId">Id of Activity</param>
		public ValidationResult(bool success, TraceInfo traceInfo, Guid activityId)
		{
			this.ValidationSuccess = success;
			this.TraceInfo = traceInfo;
			this.ActivityId = activityId;
		}

		/// <summary>
		/// Validation success result
		/// </summary>
		[DataMember]
		public bool ValidationSuccess { get; set; }

		/// <summary>
		/// Trace information
		/// </summary>
		[DataMember]
		public TraceInfo TraceInfo { get; set; }

		/// <summary>
		/// Id of Activity.
		/// </summary>
		[DataMember]
		public Guid ActivityId { get; set; }

		#region IExtensibleDataObject Implementation
		private ExtensionDataObject _extensionDataObject;
		public ExtensionDataObject ExtensionData
		{
			get
			{
				return _extensionDataObject;
			}
			set
			{
				_extensionDataObject = value;
			}
		}
		#endregion IExtensibleDataObject Implementation
	}
}
