﻿// =====================================================================
//
//  This file is part of the Microsoft Dynamics CRM SDK code samples.
//
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  This source code is intended only as a supplement to Microsoft
//  Development Tools and/or on-line documentation.  See these other
//  materials for detailed information regarding Microsoft code samples.
//
//  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
// =====================================================================
#region

using System;
using System.Collections.Generic;
using System.Linq;
using System.Xml.Linq;
using Microsoft.Xrm.Sdk.XmlNamespaces;

#endregion

namespace Microsoft.Xrm.Sdk.Client.Protocols.Policy
{
	public sealed class UserNameTokenTemplate : TokenTemplate
	{
		public static Boolean ExtractToken(XElement policy, TokenPolicy trustPolicy)
		{
			IEnumerable<XElement> bindingElements =
				from el in policy.Descendants(PublicNamespaces.SP + @"SignedEncryptedSupportingTokens")
				select el;


			foreach (XElement bindingElement in bindingElements)
			{
				IEnumerable<XElement> usernameTokens =
					from el in bindingElement.Descendants(PublicNamespaces.SP + @"UsernameToken")
					select el;

				foreach (XElement usernameToken in usernameTokens)
				{
					UserNameTokenTemplate requestToken = new UserNameTokenTemplate();
					IEnumerable<XElement> trustElements = usernameToken.Descendants();
					foreach (XElement trustElement in trustElements)
					{
						requestToken.SetPropertyValue(trustElement.Name.LocalName, trustElement.Value);
					}
					trustPolicy.Tokens[KeyTypes.UserName] = requestToken;
					trustPolicy.CredentialType = TokenServiceCredentialType.Username;
					return true;
				}
			}
			return false;
		}
	}
}
