﻿// =====================================================================
//
//  This file is part of the Microsoft Dynamics CRM SDK code samples.
//
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  This source code is intended only as a supplement to Microsoft
//  Development Tools and/or on-line documentation.  See these other
//  materials for detailed information regarding Microsoft code samples.
//
//  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
// =====================================================================
using System.Runtime.Serialization;

namespace Microsoft.Xrm.Sdk.Metadata
{
	[DataContract(Name = "BooleanAttributeMetadata", Namespace = XmlNamespaces.V5.Metadata)]
	public sealed class BooleanAttributeMetadata : AttributeMetadata
	{
		private BooleanOptionSetMetadata _boolOptionSet;
		private bool? _defaultValue;

		public BooleanAttributeMetadata()
			: this(null, null)
		{
		}

		public BooleanAttributeMetadata(string schemaName)
			: this(schemaName, null)
		{
		}

		public BooleanAttributeMetadata(BooleanOptionSetMetadata optionSet)
			: this(null, optionSet)
		{
		}

		public BooleanAttributeMetadata(string schemaName, BooleanOptionSetMetadata optionSet)
			: base(AttributeTypeCode.Boolean, schemaName)
		{
			this.OptionSet = optionSet;
		}

		[DataMember]
		public bool? DefaultValue
		{
			get { return this._defaultValue; }
			set { this._defaultValue = value; }
		}

		[DataMember]
		public BooleanOptionSetMetadata OptionSet
		{
			get { return this._boolOptionSet; }
			set { this._boolOptionSet = value; }
		}
	}
}
