﻿// =====================================================================
//
//  This file is part of the Microsoft Dynamics CRM SDK code samples.
//
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  This source code is intended only as a supplement to Microsoft
//  Development Tools and/or on-line documentation.  See these other
//  materials for detailed information regarding Microsoft code samples.
//
//  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
// =====================================================================
using System.Runtime.Serialization;

namespace Microsoft.Xrm.Sdk.Metadata
{
	[DataContract(Name = "IntegerAttributeMetadata", Namespace = XmlNamespaces.V5.Metadata)]
	public sealed class IntegerAttributeMetadata : AttributeMetadata
	{
		#region Constants
		public const int MinSupportedValue = int.MinValue;
		public const int MaxSupportedValue = int.MaxValue;
		#endregion

		#region Fields
		private IntegerFormat? _format;
		private int? _maxValue;
		private int? _minValue;
		#endregion

		#region Constructors
		public IntegerAttributeMetadata()
			: this(null)
		{
		}

		public IntegerAttributeMetadata(string schemaName)
			: base(AttributeTypeCode.Integer, schemaName)
		{
		}
		#endregion

		#region Properties
		/// <summary>
		/// Required for CreateAttribute
		/// Ignored for UpdateAttribute
		/// </summary>
		[DataMember]
		public IntegerFormat? Format
		{
			get { return this._format; }
			set { this._format = value; }
		}

		/// <summary>
		/// Required for CreateAttribute
		/// Valid for UpdateAttribute
		/// </summary>
		[DataMember]
		public int? MaxValue
		{
			get { return this._maxValue; }
			set { this._maxValue = value; }
		}

		/// <summary>
		/// Required for CreateAttribute
		/// Valid for UpdateAttribute
		/// </summary>
		[DataMember]
		public int? MinValue
		{
			get { return this._minValue; }
			set { this._minValue = value; }
		}
		#endregion
	}
}
