﻿// =====================================================================
//
//  This file is part of the Microsoft Dynamics CRM SDK code samples.
//
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  This source code is intended only as a supplement to Microsoft
//  Development Tools and/or on-line documentation.  See these other
//  materials for detailed information regarding Microsoft code samples.
//
//  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
// =====================================================================
using System.Runtime.Serialization;

namespace Microsoft.Xrm.Sdk.Metadata
{
	[DataContract(Name = "MemoAttributeMetadata", Namespace = XmlNamespaces.V5.Metadata)]
	public sealed class MemoAttributeMetadata : AttributeMetadata
	{
		#region Constants
		public const int MinSupportedLength = 1;
		public const int MaxSupportedLength = 1048576;	// One megabyte of characters (two megabytes of storage for double-byte characters)
		#endregion

		#region Properties
		private StringFormat? _format;
		private ImeMode? _imeMode;
		private int? _maxLength;
		#endregion

		#region Constructors
		public MemoAttributeMetadata()
			: this(null)
		{
		}

		public MemoAttributeMetadata(string schemaName)
			: base(AttributeTypeCode.Memo, schemaName)
		{
		}
		#endregion

		#region Properties
		/// <summary>
		/// Valid for CreateAttribute
		/// Valid for UpdateAttribute
		/// </summary>
		[DataMember]
		public StringFormat? Format
		{
			get { return this._format; }
			set { this._format = value; }
		}

		/// <summary>
		/// Valid for CreateAttribute
		/// Valid for UpdateAttribute
		/// </summary>
		[DataMember]
		public ImeMode? ImeMode
		{
			get { return this._imeMode; }
			set { this._imeMode = value; }
		}

		/// <summary>
		/// Required on non-email Memo attributes for CreateAttribute
		/// Valid on UpdateAttribute
		/// </summary>
		[DataMember]
		public int? MaxLength
		{
			get { return this._maxLength; }
			set { this._maxLength = value; }
		}
		#endregion
	}
}
