﻿// =====================================================================
//
//  This file is part of the Microsoft Dynamics CRM SDK code samples.
//
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  This source code is intended only as a supplement to Microsoft
//  Development Tools and/or on-line documentation.  See these other
//  materials for detailed information regarding Microsoft code samples.
//
//  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
// =====================================================================
using System.Runtime.Serialization;

namespace Microsoft.Xrm.Sdk.Metadata
{
	[DataContract(Name = "StringAttributeMetadata", Namespace = XmlNamespaces.V5.Metadata)]
	public sealed class StringAttributeMetadata : AttributeMetadata
	{
		#region Constants
		public const int MinSupportedLength = 1;
		public const int MaxSupportedLength = 4000;
		#endregion

		#region Fields
		private StringFormat? _format;
		private ImeMode? _imeMode;
		private int? _maxLength;
		private string _yomiOf;

		private int? _databaseLength;
		#endregion

		#region Constructors
		public StringAttributeMetadata()
			: this(null)
		{
		}

		public StringAttributeMetadata(string schemaName)
			: base(AttributeTypeCode.String, schemaName)
		{
		}
		#endregion

		#region Properties
		/// <summary>
		/// Required for CreateAttribute.  
		/// Ignored for UpdateAttribute.
		/// </summary>
		[DataMember]
		public StringFormat? Format
		{
			get { return this._format; }
			set { this._format = value; }
		}

		/// <summary>
		/// Valid for CreateAttribute.
		/// Valid for UpdateAttribute.
		/// </summary>
		[DataMember]
		public ImeMode? ImeMode
		{
			get { return this._imeMode; }
			set { this._imeMode = value; }
		}

		/// <summary>
		/// Required for CreateAttribu;te
		/// Valid for UpdateAttribute
		/// </summary>
		[DataMember]
		public int? MaxLength
		{
			get { return this._maxLength; }
			set { this._maxLength = value; }
		}

		/// <summary>
		/// Required for CreateAttribute
		/// Valid for UpdateAttribute
		/// </summary>
		[DataMember]
		public string YomiOf
		{
			get { return this._yomiOf; }
			set { this._yomiOf = value; }
		}

		internal int? DatabaseLength
		{
			get { return _databaseLength; }
			set { _databaseLength = value; }
		}
		#endregion
	}
}
