// =====================================================================
//
//  This file is part of the Microsoft Dynamics CRM SDK code samples.
//
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  This source code is intended only as a supplement to Microsoft
//  Development Tools and/or on-line documentation.  See these other
//  materials for detailed information regarding Microsoft code samples.
//
//  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
// =====================================================================
namespace Microsoft.Xrm.Sdk.Query
{
    using System.Runtime.Serialization;
	using Microsoft.Xrm.Sdk.Temporary;

	/// <summary>
	/// Allow specifying selected columns
	/// </summary>
	[DataContract(Name = "ColumnSet", Namespace = XmlNamespaces.V5.Contracts)]
	public sealed class ColumnSet : IExtensibleDataObject
	{

		public ColumnSet()
		{
		}

		public ColumnSet(bool allColumns)
		{
			_allColumns = allColumns;
		}
		public ColumnSet(params string[] columns)
		{
			_columns = new DataCollection<string>(columns);
		}

		public void AddColumns(params string[] columns)
		{
			foreach (string column in columns)
			{
				this.Columns.Add(column);
			}
		}

		public void AddColumn(string column)
		{
			this.Columns.Add(column);
		}

		[DataMember]
		public bool AllColumns
		{
			get
			{
				return _allColumns;
			}
			set
			{
				_allColumns = value;
			}
		}

		[DataMember]
		public DataCollection<string> Columns
		{
			get
			{
				if (_columns == null)
				{
					_columns = new DataCollection<string>();
				}
				return _columns;
			}
		}

		[System.ComponentModel.EditorBrowsable(System.ComponentModel.EditorBrowsableState.Never)]
		internal void Accept(IQueryVisitor visitor)
		{
			visitor.Visit(this);
		}

		private bool _allColumns;
		private DataCollection<string> _columns;

		#region IExtensibleDataObject Implementation
		private ExtensionDataObject ExtensionDataObject;
		public ExtensionDataObject ExtensionData
		{
			get
			{
				return ExtensionDataObject;
			}
			set
			{
				ExtensionDataObject = value;
			}
		}
		#endregion IExtensibleDataObject Implementation
	}
}

