// =====================================================================
//
//  This file is part of the Microsoft Dynamics CRM SDK code samples.
//
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  This source code is intended only as a supplement to Microsoft
//  Development Tools and/or on-line documentation.  See these other
//  materials for detailed information regarding Microsoft code samples.
//
//  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
// =====================================================================
namespace Microsoft.Xrm.Sdk.Query
{
	using System.Runtime.Serialization;
	using Microsoft.Xrm.Sdk.Temporary;

	[DataContract(Name = "PagingInfo", Namespace = XmlNamespaces.V5.Contracts)]
	public sealed class PagingInfo : IExtensibleDataObject
	{
		/// <summary>
		/// The page to be returned.
		/// </summary>
		[DataMember]
		public int PageNumber
		{
			get
			{
				return _pageNumber;
			}
			set
			{
				_pageNumber = value;
			}
		}

		/// <summary>
		/// number of rows per page.
		/// </summary>
		[DataMember]
		public int Count
		{
			get
			{
				return _count;
			}
			set
			{
				_count = value;
			}
		}

		/// <summary>
		/// If total record count across all pages is required
		/// </summary>
		[DataMember]
		public bool ReturnTotalRecordCount
		{
			get { return _returnTotalRecordCount; }
			set { _returnTotalRecordCount = value; }
		}

		/// <summary>
		/// Page Cookie information, null indicates old paging to be used.
		/// </summary>
		[DataMember]
		public string PagingCookie
		{
			get
			{
				return _pagingCookie;
			}
			set
			{
				_pagingCookie = value;
			}
		}

		[System.ComponentModel.EditorBrowsable(System.ComponentModel.EditorBrowsableState.Never)]
		internal void Accept(IQueryVisitor visitor)
		{
			visitor.Visit(this);
		}

		#region private members
		private int _pageNumber;
		private int _count;
		private string _pagingCookie;
		private bool _returnTotalRecordCount;
		#endregion

		#region IExtensibleDataObject Implementation
		private ExtensionDataObject ExtensionDataObject;
		public ExtensionDataObject ExtensionData
		{
			get
			{
				return ExtensionDataObject;
			}
			set
			{
				ExtensionDataObject = value;
			}
		}
		#endregion IExtensibleDataObject Implementation
	}
}

