﻿// =====================================================================
//
//  This file is part of the Microsoft Dynamics CRM SDK code samples.
//
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  This source code is intended only as a supplement to Microsoft
//  Development Tools and/or on-line documentation.  See these other
//  materials for detailed information regarding Microsoft code samples.
//
//  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
// =====================================================================

using System;
using SL.IdentityModel.Claims;

namespace SL.IdentityModel.Services
{
	/// <summary>
	/// Class us used to be returned by an asyc call retieve user claims information
	/// </summary>
	public class ClaimsIdentityEventArgs : EventArgs
	{
		private Exception _error = null;
		private IClaimsPrincipal _user;

		/// <summary>
		/// Set to an exception if an error occured, otherwise is null
		/// </summary>
		public Exception Error
		{
			get { return _error; }
		}

		/// <summary>
		/// Returns IClaimsPrincipal with users claims identity
		/// </summary>
		public IClaimsPrincipal User
		{
			get { return _user; }
		}

		/// <summary>
		/// Creates an instance of the class
		/// </summary>
		/// <param name="user">sets User</param>
		/// <param name="error">sets Error</param>
		public ClaimsIdentityEventArgs(IClaimsPrincipal user, Exception error)
		{
			_error = error;
			_user = user;
		}
	}
}
